@@ -1,6 +1,6 @@
 intercept<-function (ogg, parm, gap=TRUE, rev.sgn = FALSE, var.diff = FALSE, 
     digits = max(3, getOption("digits") - 3)){
-#corregge in caso di no model intercept -- CHE VOLEVO DIRE?? #forse che adesso funziona se nel modello non c' l'interc.
+#corregge in caso di no model intercept -- CHE VOLEVO DIRE?? #forse che adesso funziona se nel modello non c'e' l'interc.
 #--
         f.U<-function(nomiU, term=NULL){
         #trasforma i nomi dei coeff U (o V) nei nomi delle variabili corrispondenti
@@ -72,6 +72,7 @@ intercept<-function (ogg, parm, gap=TRUE, rev.sgn = FALSE, var.diff = FALSE,
         }
         #if(gap) alpha<-alpha -gapCoef[grep(paste("\\.",nomeZ[i],"$",sep=""), rownames(gapCoef), value = FALSE),"Est."]
         cof.out <- c(alpha00, alpha)
+        if(rev.sgn[i]) cof.out <- cof.out[length(cof.out):1]
         ris <- matrix(cof.out)
         dimnames(ris) <- list(paste("intercept", 1:nrow(ris), sep = ""), "Est.")
         Ris[[nomeZ[i]]] <- signif(ris, digits)
